@@ -16,7 +16,13 @@ impl Draw for Model {
 impl Widget for Model {}
 
 fn main() {
-    let term: Term<()> = Term::with_height(TermHeight::Percent(50)).unwrap();
+    let term: Term<()> = Term::with_options(
+        TermOptions::default()
+            .height(TermHeight::Percent(50))
+            .disable_alternate_screen(true)
+            .clear_on_start(false),
+    )
+    .unwrap();
     let model = Model("Hey, I'm in middle!".to_string());
 
     while let Ok(ev) = term.poll_event() {
@@ -33,7 +39,14 @@ fn main() {
             }))
             .border(true);
 
-        let win = Win::new(&inner_win)
+        let win_bottom_title = Win::new(&inner_win)
+            .title_align(HorizontalAlign::Center)
+            .title("Title (at bottom) center aligned")
+            .right_prompt("Right Prompt stays")
+            .title_on_top(false)
+            .border_bottom(true);
+
+        let win = Win::new(&win_bottom_title)
             .margin(Size::Percent(10))
             .padding(1)
             .title("Window Title")
