@@ -5,5 +5,8 @@ pub use crate::draw::{Draw, DrawResult};
 pub use crate::event::Event;
 pub use crate::key::*;
 pub use crate::term::{Term, TermHeight, TermOptions};
-pub use crate::widget::{HSplit, Rectangle, Size, Split, Stack, VSplit, Widget, Win};
+pub use crate::widget::{
+    AlignSelf, HSplit, HorizontalAlign, Rectangle, Size, Split, Stack, VSplit, VerticalAlign,
+    Widget, Win,
+};
 pub use crate::Result;
