@@ -1,4 +1,5 @@
 use super::split::Split;
+use super::util::adjust_event;
 use super::Size;
 use super::{Rectangle, Widget};
 use crate::attr::Attr;
@@ -6,8 +7,8 @@ use crate::canvas::{BoundedCanvas, Canvas};
 use crate::cell::Cell;
 use crate::draw::{Draw, DrawResult};
 use crate::event::Event;
-use crate::key::Key;
-use crate::unwrap_or_return;
+use crate::widget::align::{AlignSelf, HorizontalAlign};
+use crate::{ok_or_return, some_or_return};
 use std::cmp::max;
 use unicode_width::UnicodeWidthStr;
 
@@ -40,6 +41,8 @@ pub struct Win<'a, Message = ()> {
     title_attr: Attr,
     right_prompt: Option<String>,
     right_prompt_attr: Attr,
+    title_align: HorizontalAlign,
+    title_on_top: bool,
 
     basis: Size,
     grow: usize,
@@ -73,6 +76,8 @@ impl<'a, Message> Win<'a, Message> {
             title_attr: Default::default(),
             right_prompt: None,
             right_prompt_attr: Default::default(),
+            title_align: HorizontalAlign::Left,
+            title_on_top: true,
             basis: Size::Default,
             grow: 1,
             shrink: 1,
@@ -220,6 +225,16 @@ impl<'a, Message> Win<'a, Message> {
         self
     }
 
+    pub fn title_align(mut self, align: HorizontalAlign) -> Self {
+        self.title_align = align;
+        self
+    }
+
+    pub fn title_on_top(mut self, title_on_top: bool) -> Self {
+        self.title_on_top = title_on_top;
+        self
+    }
+
     pub fn basis(mut self, basis: impl Into<Size>) -> Self {
         self.basis = basis.into();
         self
@@ -269,10 +284,21 @@ impl<'a, Message> Win<'a, Message> {
             height,
         } = rect_reserve_margin;
 
-        let height_needed = if self.border_bottom { 2 } else { 1 };
+        let new_top = if self.title_on_top {
+            top
+        } else {
+            max(top + height, 1) - 1
+        };
+
+        let height_needed = if self.title_on_top && self.border_bottom {
+            2
+        } else {
+            1
+        };
         if height_needed > height {
+            // not enough space, don't draw at all
             return Rectangle {
-                top,
+                top: new_top,
                 left,
                 width,
                 height: 0,
@@ -289,7 +315,7 @@ impl<'a, Message> Win<'a, Message> {
         }
         if width_needed > width {
             return Rectangle {
-                top,
+                top: new_top,
                 left,
                 width: 0,
                 height,
@@ -297,7 +323,7 @@ impl<'a, Message> Win<'a, Message> {
         }
 
         Rectangle {
-            top,
+            top: new_top,
             left,
             width: width - width_needed,
             height: 1,
@@ -313,10 +339,13 @@ impl<'a, Message> Win<'a, Message> {
         } = rect;
 
         // title and right prompt will be displayed on top
-        let border_top = self.border_top || self.title.is_some() || self.right_prompt.is_some();
+        let border_top = self.border_top
+            || (self.title_on_top && (self.title.is_some() || self.right_prompt.is_some()));
+        let border_bottom = self.border_bottom
+            || (!self.title_on_top && (self.title.is_some() || self.right_prompt.is_some()));
 
-        if border_top || self.border_bottom {
-            if (height < 1) || (border_top && self.border_bottom && height < 2) {
+        if border_top || border_bottom {
+            if (height < 1) || (border_top && border_bottom && height < 2) {
                 return Err("not enough height for border".into());
             }
         }
@@ -332,11 +361,7 @@ impl<'a, Message> Win<'a, Message> {
         let width = if self.border_left { width - 1 } else { width };
         let width = if self.border_right { width - 1 } else { width };
         let height = if border_top { height - 1 } else { height };
-        let height = if self.border_bottom {
-            height - 1
-        } else {
-            height
-        };
+        let height = if border_bottom { height - 1 } else { height };
 
         Ok(Rectangle {
             top,
@@ -464,29 +489,30 @@ impl<'a, Message> Win<'a, Message> {
     }
 
     fn draw_title_and_prompt(&self, canvas: &mut dyn Canvas) -> DrawResult<()> {
-        let (width, _height) = canvas.size()?;
+        let (width, height) = canvas.size()?;
+        let row = if self.title_on_top {
+            0
+        } else {
+            max(height, 1) - 1
+        };
+
         if self.right_prompt.is_some() {
             let prompt = self.right_prompt.as_ref().unwrap();
-            let string_width = prompt.width_cjk();
-            let left = if string_width > width {
-                0
-            } else {
-                width - string_width
-            };
-            canvas.print_with_attr(0, left, prompt, self.right_prompt_attr)?;
+            let text_width = prompt.width_cjk();
+            let left = HorizontalAlign::Right.adjust(0, width, text_width);
+            canvas.print_with_attr(row, left, prompt, self.right_prompt_attr)?;
         }
 
         if self.title.is_some() {
             let title = self.title.as_ref().unwrap();
-
-            canvas.print_with_attr(0, 0, title, self.right_prompt_attr)?;
+            let text_width = title.width_cjk();
+            let left = self.title_align.adjust(0, width, text_width);
+            canvas.print_with_attr(row, left, title, self.right_prompt_attr)?;
         }
 
         Ok(())
     }
 
-    /// draw border and return the position & size of the inner canvas
-    /// (top, left, width, height)
     fn draw_header(&self, canvas: &mut dyn Canvas) -> DrawResult<()> {
         let (width, height) = canvas.size()?;
         if width <= 0 || height <= 0 {
@@ -501,11 +527,8 @@ impl<'a, Message> Win<'a, Message> {
 
         Ok(())
     }
-}
 
-impl<'a, Message> Draw for Win<'a, Message> {
-    /// Reserve margin & padding, draw border.
-    fn draw(&self, canvas: &mut dyn Canvas) -> DrawResult<()> {
+    fn draw_context(&self, canvas: &'a mut dyn Canvas) -> DrawResult<BoundedCanvas<'a>> {
         let (width, height) = canvas.size()?;
         let outer_rect = Rectangle {
             top: 0,
@@ -533,9 +556,21 @@ impl<'a, Message> Draw for Win<'a, Message> {
             height,
         } = self.calc_inner_rect(outer_rect)?;
 
-        let mut new_canvas = BoundedCanvas::new(top, left, width, height, canvas);
+        Ok(BoundedCanvas::new(top, left, width, height, canvas))
+    }
+}
+
+impl<'a, Message> Draw for Win<'a, Message> {
+    /// Reserve margin & padding, draw border.
+    fn draw(&self, canvas: &mut dyn Canvas) -> DrawResult<()> {
+        let mut new_canvas = self.draw_context(canvas)?;
         self.inner.draw(&mut new_canvas)
     }
+
+    fn draw_mut(&mut self, canvas: &mut dyn Canvas) -> DrawResult<()> {
+        let mut new_canvas = self.draw_context(canvas)?;
+        self.inner.draw_mut(&mut new_canvas)
+    }
 }
 
 impl<'a, Message> Widget<Message> for Win<'a, Message> {
@@ -559,68 +594,15 @@ impl<'a, Message> Widget<Message> for Win<'a, Message> {
 
     fn on_event(&self, event: Event, rect: Rectangle) -> Vec<Message> {
         let empty = vec![];
-        let inner_rect = unwrap_or_return!(self.calc_inner_rect(rect), empty);
-
-        let adjusted_event = match event {
-            Event::Key(Key::MousePress(button, row, col)) => {
-                if inner_rect.contains(row as usize, col as usize) {
-                    let (row, col) = inner_rect.relative_to_origin(row as usize, col as usize);
-                    Event::Key(Key::MousePress(button, row as u16, col as u16))
-                } else {
-                    return empty;
-                }
-            }
-            Event::Key(Key::MouseRelease(row, col)) => {
-                if inner_rect.contains(row as usize, col as usize) {
-                    let (row, col) = inner_rect.relative_to_origin(row as usize, col as usize);
-                    Event::Key(Key::MouseRelease(row as u16, col as u16))
-                } else {
-                    return empty;
-                }
-            }
-            Event::Key(Key::MouseHold(row, col)) => {
-                if inner_rect.contains(row as usize, col as usize) {
-                    let (row, col) = inner_rect.relative_to_origin(row as usize, col as usize);
-                    Event::Key(Key::MouseHold(row as u16, col as u16))
-                } else {
-                    return empty;
-                }
-            }
-            Event::Key(Key::SingleClick(button, row, col)) => {
-                if inner_rect.contains(row as usize, col as usize) {
-                    let (row, col) = inner_rect.relative_to_origin(row as usize, col as usize);
-                    Event::Key(Key::SingleClick(button, row as u16, col as u16))
-                } else {
-                    return empty;
-                }
-            }
-            Event::Key(Key::DoubleClick(button, row, col)) => {
-                if inner_rect.contains(row as usize, col as usize) {
-                    let (row, col) = inner_rect.relative_to_origin(row as usize, col as usize);
-                    Event::Key(Key::DoubleClick(button, row as u16, col as u16))
-                } else {
-                    return empty;
-                }
-            }
-            Event::Key(Key::WheelDown(row, col, count)) => {
-                if inner_rect.contains(row as usize, col as usize) {
-                    let (row, col) = inner_rect.relative_to_origin(row as usize, col as usize);
-                    Event::Key(Key::WheelDown(row as u16, col as u16, count))
-                } else {
-                    return empty;
-                }
-            }
-            Event::Key(Key::WheelUp(row, col, count)) => {
-                if inner_rect.contains(row as usize, col as usize) {
-                    let (row, col) = inner_rect.relative_to_origin(row as usize, col as usize);
-                    Event::Key(Key::WheelUp(row as u16, col as u16, count))
-                } else {
-                    return empty;
-                }
-            }
-            ev => ev,
-        };
+        let inner_rect = ok_or_return!(self.calc_inner_rect(rect), empty);
+        let adjusted_event = some_or_return!(adjust_event(event, inner_rect), empty);
+        self.inner.on_event(adjusted_event, inner_rect)
+    }
 
+    fn on_event_mut(&mut self, event: Event, rect: Rectangle) -> Vec<Message> {
+        let empty = vec![];
+        let inner_rect = ok_or_return!(self.calc_inner_rect(rect), empty);
+        let adjusted_event = some_or_return!(adjust_event(event, inner_rect), empty);
         self.inner.on_event(adjusted_event, inner_rect)
     }
 }
@@ -643,6 +625,7 @@ impl<'a, Message> Split<Message> for Win<'a, Message> {
 #[allow(dead_code)]
 mod test {
     use super::*;
+    use std::sync::Mutex;
 
     struct WinHint {
         pub width_hint: Option<usize>,
@@ -702,4 +685,75 @@ mod test {
         let win_border_left = Win::new(&inner).border_left(true);
         assert_eq!((None, Some(1)), win_border_left.size_hint());
     }
+
+    #[derive(PartialEq, Debug)]
+    enum Called {
+        No,
+        Mut,
+        Immut,
+    }
+
+    struct Drawn {
+        called: Mutex<Called>,
+    }
+
+    impl Draw for Drawn {
+        fn draw(&self, _canvas: &mut dyn Canvas) -> DrawResult<()> {
+            *self.called.lock().unwrap() = Called::Immut;
+            Ok(())
+        }
+        fn draw_mut(&mut self, _canvas: &mut dyn Canvas) -> DrawResult<()> {
+            *self.called.lock().unwrap() = Called::Mut;
+            Ok(())
+        }
+    }
+
+    impl Widget for Drawn {}
+
+    #[derive(Default)]
+    struct TestCanvas {}
+
+    #[allow(unused_variables)]
+    impl Canvas for TestCanvas {
+        fn size(&self) -> crate::Result<(usize, usize)> {
+            Ok((100, 100))
+        }
+
+        fn clear(&mut self) -> crate::Result<()> {
+            unimplemented!()
+        }
+
+        fn put_cell(&mut self, row: usize, col: usize, cell: Cell) -> crate::Result<usize> {
+            Ok(1)
+        }
+
+        fn set_cursor(&mut self, row: usize, col: usize) -> crate::Result<()> {
+            unimplemented!()
+        }
+
+        fn show_cursor(&mut self, show: bool) -> crate::Result<()> {
+            unimplemented!()
+        }
+    }
+
+    #[test]
+    fn mutable_widget() {
+        let mut canvas = TestCanvas::default();
+
+        let mut mutable = Drawn {
+            called: Mutex::new(Called::No),
+        };
+        {
+            let mut win = Win::new(&mut mutable);
+            let _ = win.draw_mut(&mut canvas).unwrap();
+        }
+        assert_eq!(Called::Mut, *mutable.called.lock().unwrap());
+
+        let immutable = Drawn {
+            called: Mutex::new(Called::No),
+        };
+        let win = Win::new(&immutable);
+        let _ = win.draw(&mut canvas).unwrap();
+        assert_eq!(Called::Immut, *immutable.called.lock().unwrap());
+    }
 }
